"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.throttle = throttle;
var _effector = require("effector");
function throttle({
  source,
  timeout,
  target = (0, _effector.createEvent)({
    name: "target",
    sid: "r328r6"
  })
}) {
  if (!_effector.is.unit(source)) throw new TypeError('source must be unit from effector');
  const $timeout = toStoreNumber(timeout);
  const timerFx = (0, _effector.createEffect)({
    name: `throttle(${source.shortName || source.kind}) effect`,
    handler: timeout => new Promise(resolve => setTimeout(resolve, timeout))
  }, {
    name: "timerFx",
    sid: "-pyjx8k"
  });

  // It's ok - nothing will ever start unless source is triggered
  const $payload = (0, _effector.createStore)(null, {
    and: {
      serialize: 'ignore',
      skipVoid: false
    },
    name: "$payload",
    sid: "hzp5u0"
  }).on(source, (_, payload) => payload);
  const triggerTick = (0, _effector.createEvent)({
    name: "triggerTick",
    sid: "-ry76an"
  });
  const $canTick = (0, _effector.createStore)(true, {
    and: {
      serialize: 'ignore'
    },
    name: "$canTick",
    sid: "-4yqc5"
  }).on(triggerTick, () => false).on(target, () => true);
  (0, _effector.sample)({
    and: [{
      clock: source,
      filter: $canTick,
      target: triggerTick
    }],
    or: {
      sid: "-hyxqny"
    }
  });
  (0, _effector.sample)({
    and: [{
      source: $timeout,
      clock: triggerTick,
      target: timerFx
    }],
    or: {
      sid: "-hk3an7"
    }
  });
  (0, _effector.sample)({
    and: [{
      source: $payload,
      clock: timerFx.done,
      target
    }],
    or: {
      sid: "-hgsj31"
    }
  });
  return target;
}
function toStoreNumber(value) {
  if (_effector.is.store(value, {
    sid: "-gzqwle"
  })) return value;
  if (typeof value === 'number') {
    if (value < 0 || !Number.isFinite(value)) throw new Error(`timeout must be positive number or zero. Received: "${value}"`);
    return (0, _effector.createStore)(value, {
      and: {
        name: '$timeout'
      },
      sid: "-gs60mt"
    });
  }
  throw new TypeError(`timeout parameter should be number or Store. "${typeof value}" was passed`);
}